param nauctions integer;				# The number of auctions
param nbids integer;					# The number of bidders in each auction
param ndata := nbids * nauctions;		# The number of total observed bids
set T := 1..ndata; 						# The number of bids
param bids {t in T};					# The bid data
data bids450.dat;

########################################################
########### APPROXATION PARAMETERS:  ###################
########################################################
param pi := 3.141592654;
param nnodes := 200;					# The approximation nodes
param order := 6;						# The order of the approximation polynomials
param xmin := 0;
param xmax := max{t in T} bids[t];		# Indicates the upper limit on the interval of approximation
param gamma := 200;						# The penalty parameter
set J := 1..order+1;
set K := 1..nnodes;

# set of Chebyshev nodes on [xmin,xmax]:
param  x {i in K} := xmin + (xmax-xmin)/(2)*(1 - cos((2*i-1)/(2*nnodes)*pi));  # set of Chebyshev state variables
var a {j in J};							# The wieghts on the basis polynomials
fix a[1] := 0;
#fix a[2] := 1;

var lambda { k in K} >=0;				# The error terms

#########################################################
########### DISTRIBUTION PARAMETERS:  ###################
#########################################################
var theta >= 0;
let theta := .05;
# var z >= xmax*(1/.95); # See feasibility constraint below

########################################################
########### APPROXATION FUNCTIONS:   ###################
########################################################
# Use the standard basis polynomials and here we construct the weighted sum at the Cheb nodes
var invbid {k in K} = sum { j in J} a[j]*x[k]^(j-1);
var invbidprime {k in K} = sum { j in J: j >=2} a[j]*(j-1)*x[k]^(j-2); 

# Distribution and density
var Fv { k in K} = 1 - exp(- invbid[k]*theta);
var fv { k in K} = theta * exp(-invbid[k]*theta);
var v {t in T} =  sum { j in J} a[j]*bids[t]^(j-1);

display x;
# Approximation residuals at Cheb nodes
var residual {k in K} = invbidprime[k] * (invbid[k]-x[k])*fv[k] - Fv[k];
var LL =  ndata*log(theta) - (theta) * sum{t in T} v[t];

########################################################
###########      OPTIMIZATION:       ###################
########################################################
minimize objective:  -LL + sum {k in K} gamma* lambda[k] ;
subject to FIRST_MOMENT :  1/theta= (1/ndata * sum {t in T} v[t]);
subject to LOWER_ERROR_BOUND {k in K} : -lambda[k] <= residual[k];
subject to UPPER_ERROR_BOUND {k in K} : residual[k] <= lambda[k];
subject to RATIONALITY {k in K} : x[k] <= invbid[k];
subject to NONDECREASING1  {k in K:k>=2} : sum {j in J} a[j] * (x[k]^(j-1) - x[k-1]^(j-1) ) >=  0  ;
subject to NONDECREASING2 {k in K:x[k]>=1} : sum {j in J:j>=2} a[j] * (j-1)* x[k]^(j-2) >= 0 ;
# subject to FEASIBILITY : z = mu + beta*gamma; # See definition of variable z above


option solver knitro;
option knitro_options 'alg=1 outlev=1 opttol=1.0e-6 feastol=1.0e-6 maxit=25000', solver "C:\Program Files\Ziena\solvers\knitroampl-5.2.0\knitroampl\knitroampl";	

solve;

########################################################
###########         OUTPUT:          ###################
########################################################
display bids,v;

display a, theta;
display max {k in K} lambda[k];
display sum {k in K} lambda[k];
display 1/ndata * sum {t in T} v[t],  1/theta;

display LL;